function traj = SSATO(m)
% function SSATO generates single trajectories for model discrimination in 
% Figure 4 of the paper. The input m indicates whether the sample
% trajectory comes from M_1, M_2, or M_3.
if nargin < 1
    m = 1;
end;

% Rates from Hayot, Science Signalling, 2011
R = [1e-2 5.8e-3 6e-3 1.92e-4];

%set rates
if m == 1
    k = R; tmax=2000;
elseif m == 2
    k = [R(1) 10*R(2) R(3) 10*R(4)]; tmax = 500;
else
    k = [10*R(1) R(2) 10*R(3) R(4)]; tmax=5000;
end;


%set initial state (set D == 1, it does not change throughout)
M = 10; P = 0;

% run Gillespie algorithm
t = 0;
traj = [t M P];
while t < tmax
    rates = [k(1) M*k(2) M*k(3) P*k(4)];
    tau = exprnd(1./sum(rates)); t = t + tau;
    I = cumsum(rates)/sum(rates);
    r = find(rand < I,1,'first');
    if r == 1
        M = M + 1;
    elseif r == 2
        P = P + 1;
    elseif r == 3
        M = M - 1;
    else
        P = P - 1;
    end;
    traj = [traj; t M P];
end

end