% This code generates a stochastic simulation run for a colony that starts
% with one cell at time zero and each cell divides every 20 minutes, ending
% up with 8 cells and 80 minutes of data. It is used in Figure 3. The
% trajectory in colony.mat was created using this file.

t0 = [0 20 40 60]*60;
tf = [20 40 60 80]*60;

M = NaN(8,4); M(4,1) = 10;
P = NaN(8,4); P(4,1) = 0;

out{4}.traj = schnitz(t0(1),tf(1),M(4,1),P(4,1));
out{4}.Pobs = [];
for i=t0(1):60:tf(1)
   ix = find(out{4}.traj(:,1) <= i,1,'last' );
   out{4}.Pobs = [out{4}.Pobs out{4}.traj(ix,3)];
end;

% divide cell 4
foo = out{4}.traj(end,2:3);
bar = binornd(foo,.5);
M(4,2) = bar(1); P(4,2) = bar(2);
M(5,2) = foo(1) - bar(1); P(5,2) = foo(2)-bar(2);

out{4}.traj = [out{4}.traj; schnitz(t0(2),tf(2),M(4,2),P(4,2))];
out{5}.traj = schnitz(t0(2),tf(2),M(5,2),P(5,2));

out{5}.Pobs = [];
for i=(t0(2) + 60):60:tf(2)
   for j = 4:5
       ix = find(out{j}.traj(:,1) <= i,1,'last' );
       out{j}.Pobs = [out{j}.Pobs out{j}.traj(ix,3)];
   end;
end;


% divide cell 4
foo = out{4}.traj(end,2:3);
bar = binornd(foo,.5);
M(4,3) = bar(1); P(4,3) = bar(2);
M(2,3) = foo(1) - bar(1); P(2,3) = foo(2)-bar(2);

% divide cell 5
foo = out{5}.traj(end,2:3);
bar = binornd(foo,.5);
M(5,3) = bar(1); P(5,3) = bar(2);
M(7,3) = foo(1) - bar(1); P(7,3) = foo(2)-bar(2);

out{4}.traj = [out{4}.traj; schnitz(t0(3),tf(3),M(4,3),P(4,3))];
out{5}.traj = [out{5}.traj; schnitz(t0(3),tf(3),M(5,3),P(5,3))];
out{2}.traj = schnitz(t0(3),tf(3),M(2,3),P(2,3));
out{7}.traj = schnitz(t0(3),tf(3),M(7,3),P(7,3));

out{2}.Pobs = [];
out{7}.Pobs = [];
for i=(t0(3) + 60):60:tf(3)
   for j = [2 4 5 7]
       ix = find(out{j}.traj(:,1) <= i,1,'last' );
       out{j}.Pobs = [out{j}.Pobs out{j}.traj(ix,3)];
   end;
end;

% divide cell 4
foo = out{4}.traj(end,2:3);
bar = binornd(foo,.5);
M(4,4) = bar(1); P(4,4) = bar(2);
M(3,4) = foo(1) - bar(1); P(3,4) = foo(2)-bar(2);

% divide cell 5
foo = out{5}.traj(end,2:3);
bar = binornd(foo,.5);
M(5,4) = bar(1); P(5,4) = bar(2);
M(6,4) = foo(1) - bar(1); P(6,4) = foo(2)-bar(2);

% divide cell 2
foo = out{2}.traj(end,2:3);
bar = binornd(foo,.5);
M(2,4) = bar(1); P(2,4) = bar(2);
M(1,4) = foo(1) - bar(1); P(1,4) = foo(2)-bar(2);

% divide cell 7
foo = out{7}.traj(end,2:3);
bar = binornd(foo,.5);
M(7,4) = bar(1); P(7,4) = bar(2);
M(8,4) = foo(1) - bar(1); P(8,4) = foo(2)-bar(2);

out{4}.traj = [out{4}.traj; schnitz(t0(4),tf(4),M(4,4),P(4,4))];
out{5}.traj = [out{5}.traj; schnitz(t0(4),tf(4),M(5,4),P(5,4))];
out{2}.traj = [out{2}.traj; schnitz(t0(4),tf(4),M(2,4),P(2,4))];
out{7}.traj = [out{7}.traj; schnitz(t0(4),tf(4),M(7,4),P(7,4))];
out{1}.traj = schnitz(t0(4),tf(4),M(1,4),P(1,4));
out{3}.traj = schnitz(t0(4),tf(4),M(3,4),P(3,4));
out{6}.traj = schnitz(t0(4),tf(4),M(6,4),P(6,4));
out{8}.traj = schnitz(t0(4),tf(4),M(8,4),P(8,4));


out{1}.Pobs = [];
out{3}.Pobs = [];
out{6}.Pobs = [];
out{8}.Pobs = [];
for i=(t0(4) + 60):60:tf(4)
   for j = 1:8
       ix = find(out{j}.traj(:,1) <= i,1,'last' );
       out{j}.Pobs = [out{j}.Pobs out{j}.traj(ix,3)];
   end;
end;

figure
for i=1:8
subplot(8,1,i)
stairs(out{i}.traj(:,1)/60,out{i}.traj(:,3))
xlim([ 0 80 ])
ylim([ 0 30 ])
    set(gca,'YTick',[0 15 30])
    set(gca,'YTickLabel',{'0' '15' '30'})
    if i ~= 8
        set(gca,'XTick',[])
    end;
end

figure
for i=1:8
subplot(8,1,i)
stairs(out{i}.traj(:,1)/60,out{i}.traj(:,2))
xlim([ 0 80 ])
ylim([ 0 10 ])
    set(gca,'YTick',[0 5 10])
    set(gca,'YTickLabel',{'0' '5' '10'})
    if i ~= 8
        set(gca,'XTick',[])
    end;
end