% This code generates Figure 1 from the paper

% plot y vs. t to generate the output in panel (a)
% LOW = 1, MED = 2, HIGH = 3
y = [1 1 2 3 3 1 3 2 2 1 1 1 2 2 1];
t = [.1:.1:1.5];

% generate the stochastic matrix Q corresponding to the birth-death
% process
for i=1:20
    Q(i+1,i) = 8;
    Q(i,i+1) = i;
end
Q = Q - diag(sum(Q));

% define the initial conditions
p = poisspdf(0:20,8)'; t = 0;
dt = 0.01;

% initialize the forward observer
fdata = p;
time = 0;

% run the forward observer
for i=1:15
    for j=1:10
        p = expm(Q*dt)*p;
        t = t + dt;
        fdata = [fdata p];
        time = [time t];
    end;
    if y(i) == 1
        p(8:end) = 0;
        p = p/sum(p);
        fdata = [fdata p];
        time = [time t];
    elseif y(i) == 2
        p(1:7) = 0; p(15:end) = 0;
        p = p/sum(p);
        fdata = [fdata p];
        time = [time t];
    else
        p(1:14) = 0;
        p = p/sum(p);
        fdata = [fdata p];
        time = [time t];
    end;
end;

% calculate the mean and variance for the forward observer PMFs
E1 = sum(fdata .* repmat((0:20)',1,166));
SQ = ((0:20).^2)';
V1 = sum(fdata .* repmat(SQ,1,166)) - E1.^2;

% set up the backward observer
tplus = [12:11:166];
tminus = 11:11:166;

% initialize the backward observer with the final PMF from the forward
% observer
p = fdata(:,end);

bdata = [p];
btime = unique(time);


% run the backward observer
for i=15:-1:1
    Pplus = fdata(:,tplus(i))';
    Pminus = fdata(:,tminus(i))';
    if i ~= 1
        foodata = fdata(:,tplus(i-1):tminus(i));
    else
        foodata = fdata(:,1:tminus(i));
    end
    dt= .1:-.01:0;
    for j = 10:-1:1
        bar = (Pplus ./ Pminus) * expm(Q*dt(j))*diag(foodata(:,j));
        bdata = [bar' bdata];
    end;
end;

% calculate the mean and variance for the backward observer PMFs
E2 = sum(bdata .* repmat((0:20)',1,151));
V2 = sum(bdata .* repmat(SQ,1,151)) - E2.^2;