function out = identifier(traj)
% function IDENTIFIER discrimates between the three models and is used to 
% generate the plots in Figure 4 of the paper. It does so by constructing
% the stochastic matrix for each model and running the forward observer 
% on each model. There is no need to run the backward observer for model
% discrimination.


% set initial conditions;
k = [1e-2 5.8e-3 6e-3 1.92e-4];
p1 = [0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';
p2 = [0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';
p3 = [0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';
H = [1 1 1]/3;

nr = size(traj,1);
t = traj(:,1);
np = traj(:,3);

out = [0 H];
for r=2:nr
    % generate matrices for each model.
    P = traj(r-1,3);
    Q1 = zeros(25,25);
    Q2 = zeros(25,25);
    Q3 = zeros(25,25);

    for i=1:25
        % state departure rates
        Q1(i,i) = -k(1)    -(i-1)*k(2)    -(i-1)*k(3)    -P*k(4);
        Q2(i,i) = -k(1)    -10*(i-1)*k(2) -(i-1)*k(3)    -10*P*k(4);
        Q3(i,i) = -10*k(1) -(i-1)*k(2)    -10*(i-1)*k(3) -P*k(4);
    end
    for i=1:24
        % mRNA birth rates
        Q1(i+1,i) = k(1);
        Q2(i+1,i) = k(1);
        Q3(i+1,i) = 10*k(1);
        % mRNA death rates
        Q1(i,i+1) = i*k(3);
        Q2(i,i+1) = i*k(3);
        Q3(i,i+1) = 10*i*k(3);
    end;
    
    % cut off transition to nowhere.
    Q1(25,25) = Q1(25,25) + k(1);
    Q3(25,25) = Q3(25,25) + 10*k(1);
    
    % initialize models
    dt = t(r) - t(r-1);
    p1 = expm(Q1*dt)*p1;
    p2 = expm(Q2*dt)*p2;
    p3 = expm(Q3*dt)*p3;
    
    s1 = sum(p1); p1 = p1/s1;
    s2 = sum(p2); p2 = p2/s2;
    s3 = sum(p3); p3 = p3/s3;
    H = H.*[s1 s2 s3];
    H = H/sum(H);
    out = [out; t(r) H];
    
    % update probabilities of each model based on observation of changes in
    % protein number
    if np(r) == (np(r-1) + 1)
        for i=1:25
            T1(i) = (i-1)*k(2); T2(i) = 10*(i-1)*k(2); T3(i) = (i-1)*k(2);
        end;
        p1 = diag(T1)*p1; p2 = diag(T2)*p2; p3 = diag(T3)*p3;
        s1 = sum(p1); p1 = p1/s1;
        s2 = sum(p2); p2 = p2/s2;
        s3 = sum(p3); p3 = p3/s3; 
        H = H.*[s1 s2 s3];
        H = H/sum(H);
        out = [out; t(r) H];
    elseif np(r) == (np(r-1) - 1)
        U1 = P*eye(25)*k(4); U2 = P*10* eye(25)*k(4); U3 = P*eye(25)*k(4);
        p1 = U1*p1; p2 = U2*p2; p3 = U3*p3;
        s1 = sum(p1); p1 = p1/s1;
        s2 = sum(p2); p2 = p2/s2;
        s3 = sum(p3); p3 = p3/s3; 
        H = H.*[s1 s2 s3];
        H = H/sum(H);
        out = [out; t(r) H];
    end;
end;