function out = split_cells(Pr)
% Function SPLIT_CELLS takes as its input the PMF before a cell division,
% and returns as its output the PMF after cell division, assuming, that
% each protein and mRNA molecule is equally likely to join either daughter
% cell. To do this, we separate the 656-state vector Pr into the marginal
% probability distributions for P and M, because this is much faster.

P = zeros(41,1);
M = zeros(16,1);
% get marginal probabilities
for i=1:656
    [m,p] = i2s(i);
    M(m+1) = M(m+1) + Pr(i);
    P(p+1) = P(p+1) + Pr(i);
end;

% divide mRNA marginals
foo = zeros(16,1);
for i=0:15
    for j=0:i
        foo(j+1) = foo(j+1) + M(i+1) * binopdf(j,i,.5);
    end;
end;
M = foo/sum(foo);

% divide protein marginals
bar = zeros(41,1);
for i=0:40
    for j=0:i
            bar(j+1) = bar(j+1) + P(i+1) * binopdf(j,i,.5);
    end;
end;
P = bar/sum(bar);

% create output vector
out = zeros(656,1);
for m=0:15
    for p=0:40
        out(s2i(m,p)) = M(m+1)*P(p+1);
    end;
end;
end

function out = s2i(M,P)
 out = 41*M + P + 1;
end

function [M,P] = i2s(ix)
ix = ix - 1;
M = fix(ix/41);
P = rem(ix,41);
end